import os
import sys
import json
import ctypes
import socket
import pathlib
import argparse
from colorama import init, Fore, Style
from dotenv import load_dotenv, set_key

from graphenex.core.hrd import HardenMethod
from graphenex.core.utils.logcl import GraphenexLogger


logger = GraphenexLogger(__name__)
project_dir = pathlib.Path(__file__).absolute().parent.parent.parent
mod_json_file = project_dir / "modules.json"
env_file = project_dir.parent / ".env"


def parse_cli_args():
    """
    Command-line argument parser
    Returns parsed args as dict
    """

    parser = argparse.ArgumentParser(
        description='grapheneX | Automated System Hardening Framework')
    parser.add_argument('-v',
                        '--version',
                        action="store_true",
                        help="show version information")
    parser.add_argument('-w',
                        '--web',
                        action="store_true",
                        help='start the grapheneX web server')
    parser.add_argument('host_port', metavar='host:port', type=str, nargs='?',
                        default='localhost:8080',
                        help="host and port to run the web interface")
    parser.add_argument('--open', action="store_true",
                        help="open browser on web server start")
    parser.add_argument("--flask-secret-key",
                        default=None,
                        help="Secret key to keep the client-side sessions secure.")
    args = vars(parser.parse_args())
    return args


def print_header():
    """
    Shows project logo in ASCII format,
    project description and repository
    Checks dependencies for colored output
    """
    init()
    project_desc = Style.BRIGHT + Fore.WHITE + """
                 +ho:`
           `:ohh. /dddy/.
        ./ydddddd/ -hddddho:          | grapheneX |
    `:ohdddddddddds``sddddds- :.      """+Style.NORMAL+"~ Automated System Hardening Framework"+Style.BRIGHT+"""
    +ddddddddddddddh. /dds- /hdd      """+Style.NORMAL+"+ Created for Linux & Windows."+Style.BRIGHT+"""
    +dddddddddddddddd/ .. /hdddd      """+Style.NORMAL+"> https://github.com/grapheneX"+Style.BRIGHT+"""
    +ddddddddddddddddo``/hdddddd      """+Style.NORMAL+"- Copyright (C) 2019-2024"+Style.BRIGHT+"""
    +ddddddddddddddo.`+ddddddddd
    `-/+oyhddddd+``+dddddddddddd
    :o+/-.` `-` .syddddddddddddd
    +dddddddyso+:-. `.-/+oyhdddd
     -+yddddddddddddhyso/:-` `-`
        `/sddddddddddddddy+-
            -+hddddddds:`
               `/sy+-
    """+Style.NORMAL
    print(project_desc)
    logger.info("grapheneX started.")
    check_privileges()


def check_os():
    """
    Returns operating system information.
    [1] -> Windows
    [0] -> Linux (else)
    """
    return 1 if os.name == 'nt' else 0


def check_privileges():
    """Checks privileges and warns if they aren't sufficient"""

    def is_root():
        """Returns if the app is run with sudo"""
        return os.geteuid() == 0

    def is_admin():
        """Returns if the app is run with administrative access"""
        try:
            result = ctypes.windll.shell32.IsUserAnAdmin()
            return result
        except Exception:
            return False

    if check_os():
        if not is_admin():
            logger.warn("Some functions won't work without admin rights, " +
                        "try running graphenex with admin access.")
    else:
        if not is_root():
            logger.warn("Some functions won't work without root access, " +
                        "try running grapheneX with sudo.")


def get_modules():
    """Get hardening modules & namespaces in a dictionary"""

    current_os = "win" if check_os() else "linux"
    json_data = get_mod_json()
    return_dict = dict()
    available_modules = list()
    for namespace, modlist in json_data.items():
        if namespace == "presets":
            continue
        for module in modlist:
            module['namespace'] = namespace
            if module['target_os'] == current_os:
                return_dict[module['namespace']] = dict()
                available_modules.append(module)

        for module in available_modules:
            return_dict[module['namespace']
                        ][module['name']] = HardenMethod(**module)
    return return_dict


def get_forbidden_namespaces(os='win' if check_os() else 'linux'):
    """Returns the restricted namespaces depending on the operating system"""

    json_data = get_mod_json()
    namespaces = list()
    for namespace, modlist in json_data.items():
        if os not in [module['target_os'] for module in modlist]:
            namespaces.append(namespace)
    namespaces.append("presets")
    return namespaces


def get_presets(os='win' if check_os() else 'linux'):
    """Parse and return the presets in the 'modules' file"""

    try:
        presets = list()
        for preset in get_mod_json()['presets']:
            if preset['target_os'] == os:
                presets.append(preset)
        return presets
    except KeyError:
        return None


def get_mod_json():
    """Read the modules from file"""

    try:
        with open(mod_json_file, 'r') as json_file:
            json_data = json.load(json_file)
        return json_data
    except FileNotFoundError:
        logger.error(f"{mod_json_file} not found! Exiting grapheneX.")
        sys.exit(1)


def is_valid_port(port):
    """Verify the port number"""

    return 1 <= port <= 65535


def is_valid_address(address):
    """Verify the IP address"""

    try:
        socket.getaddrinfo(address, None)
        return True
    except socket.gaierror:
        return False


def create_env_file(cli_secret_key):
    """Generate .env file with secret key"""

    env_file_exists = env_file.is_file()
    if not env_file_exists:
        env_file.touch()
        logger.info("`.env` config file created")

    load_dotenv(env_file)
    flask_secret_key = os.getenv("FLASK_SECRET_KEY")

    if cli_secret_key or not flask_secret_key:
        set_key(env_file, "FLASK_SECRET_KEY", cli_secret_key or os.urandom(32).hex())


def get_flask_secret_key():
    """Load secret key from .env file"""

    load_dotenv(env_file)
    flask_secret_key = os.getenv("FLASK_SECRET_KEY")

    return flask_secret_key



import re
import psutil
import platform
from collections import namedtuple, OrderedDict
from unicodedata import normalize


class SysInformation:
    UNITS_MAPPING = [
        (1 << 50, ' PB'),
        (1 << 40, ' TB'),
        (1 << 30, ' GB'),
        (1 << 20, ' MB'),
        (1 << 10, ' KB'),
        (1, (' byte', ' bytes')),
    ]

    @staticmethod
    def pretty_size(bytes, units=UNITS_MAPPING):
        for factor, suffix in units:
            if bytes >= factor:
                break
        amount = int(bytes / factor)

        if isinstance(suffix, tuple):
            singular, multiple = suffix
            if amount == 1:
                suffix = singular
            else:
                suffix = multiple
        return str(amount) + suffix

    @staticmethod
    def get_network_info():
        masks = list()
        MaskResult = namedtuple('MaskResult', ['name', 'recv', 'sent', 'slug'])
        for mask, data in OrderedDict(psutil.net_io_counters(pernic=True)).items():
            if data.packets_recv > 0 or data.packets_sent > 0:
                res = MaskResult(mask, SysInformation.pretty_size(data.packets_recv), SysInformation.pretty_size(data.packets_sent),
                                 SysInformation.slugify(mask))
                masks.append(res)

        return masks

    @staticmethod
    def get_disk_info():
        disks = list()
        DiskResult = namedtuple('DiskResult', ['data', 'name'])
        for disk in psutil.disk_partitions():
            try:
                res = DiskResult(psutil.disk_usage(
                    disk.mountpoint), disk.mountpoint)
                disks.append(res)
            except PermissionError:
                pass

        return disks

    @staticmethod
    def get_general_info():
        uname = platform.uname()
        GeneralResult = namedtuple('GeneralResult', ['system', 'processor'])
        res = GeneralResult(f"{uname.system} | {uname.version}",
                            f"{uname.processor} - ({uname.machine})")

        return res

    @staticmethod
    def get_all_info():
        info_dict = {}
        info_dict['disks'] = SysInformation.get_disk_info()
        info_dict['network'] = SysInformation.get_network_info()
        info_dict['general'] = SysInformation.get_general_info()

        return info_dict

    @staticmethod
    def slugify(text, delim='-'):
        """
        Generate an ASCII-only slug.
        """
        _punctuation_re = re.compile(
            r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.]+')
        result = []
        for word in _punctuation_re.split(text.lower()):
            word = normalize('NFKD', word) \
                .encode('ascii', 'ignore') \
                .decode('utf-8')

            if word:
                result.append(word)

        return delim.join(result)



from cmd import Cmd

from graphenex.core.cli.commands import ShellCommands
from graphenex.core.utils.helpers import get_modules


class Shell(ShellCommands, Cmd):
    """Interactive Shell constructor class"""

    namespace = ""
    module = ""
    modules = get_modules()
    intro = "Welcome to the grapheneX interactive shell. " + \
            "Type \"help\" or \"?\" to list commands."

    @property
    def prompt(self):
        """Set prompt according to the current module and namespace"""

        prompt_str = ""
        if self.namespace:
            prompt_str = prompt_str + ":" + self.namespace
        if self.module:
            prompt_str = prompt_str + ":" + self.module
        return f"[gX{prompt_str}]> "


def start_cli():
    """Start the command line interface"""

    shell = Shell()
    try:
        shell.cmdloop()
    except KeyboardInterrupt:
        shell.do_EOF(None)



from terminaltables import AsciiTable

from graphenex.core.utils.logcl import GraphenexLogger

logger = GraphenexLogger(__name__)


class Help:
    """
    Help class that will contain help methods for specific commands
    """

    def do_help(self, arg):
        """List available commands with 'help' or show detailed help with 'help <cmd>'"""

        if arg:
            try:
                func = getattr(self, f"do_{arg}")
                if f"help_{arg}" in dir(self):
                    getattr(self, f"help_{arg}")()
                else:
                    doc = func.__doc__ if func.__doc__ else "No description"
                    print(f"\n\tSyntax: {func.__name__[3:]}\n\t{doc}\n")
            except AttributeError:
                logger.error(f"Cannot find help method for \"{arg}\".")
        else:
            # Create table for all commands
            help_table = [['Command', 'Description']]
            # In all methods and attributes
            for name in self.get_names():
                # Get do_* function
                if name[:3] == "do_" and name != "do_EOF":
                    docstr = getattr(self, name).__doc__
                    doc = docstr if docstr else "No description"
                    help_table.append([getattr(self, name).__name__[3:], doc])
            print(AsciiTable(help_table).table)

    @staticmethod
    def message(syntax, content):
        """Print the commands help message with its syntax"""

        print(f"\n\tSyntax: {syntax}\n\t{content}\n")

    def help_switch(self):
        self.message(syntax="switch [module]",
                     content="Switch between modules")

    def help_search(self):
        self.message(syntax="search [query]",
                     content="Search for module or namespace")

    def help_use(self):
        self.message(syntax="use [module]",
                     content="Use hardening module")

    def help_web(self):
        self.message(syntax="web [host:port]",
                     content="Start the web server")

    def help_preset(self):
        self.message(syntax="preset [preset]",
                     content="Execute the hardening module preset")



import socket
import secrets
import logging
import webbrowser

from flask import Flask
from flask_socketio import SocketIO

from graphenex.core.utils.helpers import is_valid_port, is_valid_address, get_flask_secret_key
from graphenex.core.cli.shell import Shell
from graphenex.core.utils.logcl import GraphenexLogger

logger = GraphenexLogger(__name__)
logging.getLogger('werkzeug').disabled = True
app = Flask(__name__)
socketio = SocketIO(app)
default_socket = ('0.0.0.0', '8080')

from graphenex.core.web.views import *  # noqa
from graphenex.core.web.providers import *  # noqa


def run_server(args=None, exit_shell=True):
    """Run the web server"""

    flask_secret_key = get_flask_secret_key()
    app.config["SECRET_KEY"] = flask_secret_key
    app.config["ACCESS_TOKEN"] = secrets.token_urlsafe(12)

    if args:
        server_params = args.get('host_port', '').split(':') or default_socket
    else:
        server_params = default_socket

    try:
        server_addr, server_port = server_params
        server_port = int(server_port)
    except (ValueError, IndexError):
        logger.error(f"Invalid socket specified: {server_params}")
        return

    if not is_valid_port(server_port):
        logger.error(f"Invalid port specified: {server_port}")
        return

    if not is_valid_address(server_addr):
        logger.error(f"Invalid address specified: {server_addr}")
        return

    logger.info(f"Starting server on http://{server_addr}:{server_port}")

    try:
        if args and args.get('open', False):
            webbrowser.open(f"http://{server_addr}:{server_port}")

        logger.info(f"Your access token: {app.config['ACCESS_TOKEN']}")
        socketio.run(app, host=server_addr, port=server_port, debug=False)
    except socket.error as e:
        if e.errno == 98:
            logger.error(f"{server_addr}:{server_port} is already in use!")
            return
    except KeyboardInterrupt:
        socketio.stop()
        if exit_shell:
            Shell().do_exit(None)
    except Exception as e:
        logger.error(f'An error occurred: {e}')
        return



import os
import re
import json
import random
import secrets
import textwrap
from terminaltables import AsciiTable

from PyInquirer import prompt, Validator, ValidationError

from graphenex.core.utils.helpers import check_os, get_modules, \
    mod_json_file, get_forbidden_namespaces, get_mod_json, get_presets
from graphenex.core.utils.logcl import GraphenexLogger
from graphenex.core.cli.help import Help

logger = GraphenexLogger(__name__)


class ShellCommands(Help):
    def do_switch(self, arg):
        """Switch between modules or namespaces"""

        if not arg:
            logger.warn("'switch' command takes 1 argument.")
            return
        arg = arg.lower()
        if arg in self.modules.keys():
            logger.info(f"Switched to \"{arg}\" namespace." +
                        " Use 'list' command to see available modules.")
            self.namespace = arg
            self.module = ""
        else:
            self.do_use(arg)

    def complete_switch(self, text, line, begidx, endidx):
        """Complete switch command"""

        avb_namespaces = [i.lower() for i in self.modules.keys()]
        mline = line.lower().partition(' ')[2]
        offs = len(mline) - len(text)
        return [s[offs:] for s in avb_namespaces if s.startswith(mline)]

    def do_use(self, arg):
        """Use the hardening module"""

        if "/" in arg and arg.split("/")[0].lower() in self.modules.keys():
            self.namespace = arg.split("/")[0].lower()
            arg = arg.split("/")[1]

        def select_module_msg():
            logger.info(f"\"{self.module}\" module selected. Use 'harden' command " +
                        "for hardening or use 'info' for more information.")

        if not arg:
            logger.warn("'use' command takes 1 argument.")
            return
        module_found = False
        if self.namespace:
            for name, module in self.modules[self.namespace].items():
                if arg.lower() == name.lower():
                    module_found = True
                    self.module = name
                    select_module_msg()
        else:
            for k, v in self.modules.items():
                for name, module in v.items():
                    if arg.lower() == name.lower():
                        module_found = True
                        self.module = name
                        self.namespace = k
                        select_module_msg()
        if not module_found:
            logger.error(f"No module/namespace named \"{arg}\".")

    def complete_use(self, text, line, begidx, endidx):
        """Complete use command"""

        avb_modules = self.modules.get(self.namespace)
        if avb_modules is None:
            avb_modules = list()
            for key, value in self.modules.items():
                for name, module in value.items():
                    avb_modules.append(f"{key}/{name}")
        mline = line.lower().partition(' ')[2]
        # If namespace selected
        if '/' in mline:
            # title() -> given module string for getting rid of
            # the case sensitivity
            mline = mline.split('/')[0].lower() + "/" + mline.split('/')[1].title()
        offs = len(mline) - len(text)
        # Get completed text with namespace
        comp_text = [s[offs:] for s in avb_modules if s.startswith(mline)]
        # If no namespace found
        if len(comp_text) == 0:
            # Try to complete with module names
            avb_modules = list()
            for key, value in self.modules.items():
                for name, module in value.items():
                    avb_modules.append(name)
            mline = mline.title()
            comp_text = [s[offs:] for s in avb_modules if s.startswith(mline)]
        return comp_text

    def do_info(self, arg):
        """Show information about the module"""

        if self.module:
            module = self.modules[self.namespace][self.module]
            print(f"\n\tNamespace: {self.namespace}\n\tModule: {module.name}\n\t" +
                  f"Description: {module.desc}\n" + f"\tCommand: {module.command}\n")
        else:
            logger.error('No module selected.')

    def do_search(self, arg):
        """Search for modules"""

        search_table = [['Module', 'Description']]
        table = AsciiTable(search_table)
        max_width = table.column_max_width(1)

        if not arg:
            self.do_list(None)
            return
        if arg in self.modules.keys():
            for name, module in self.modules[arg].items():
                wrapped = '\n'.join(textwrap.wrap(module.desc, max_width - 40))
                table.table_data.append(
                    [arg + "/" + name, wrapped])
        else:
            for k, v in self.modules.items():
                for name, module in v.items():
                    if arg.lower() in name.lower():
                        wrapped = '\n'.join(textwrap.wrap(module.desc, max_width - 40))
                        table.table_data.append(
                            [k + "/" + name, wrapped])
        if len(table.table_data) > 1:
            print(table.table)
        else:
            logger.error(f"Nothing found for \"{arg}\".")

    def do_list(self, arg):
        """List available hardening modules"""

        modules_table = [['Module', 'Description']]
        table = AsciiTable(modules_table)
        max_width = table.column_max_width(1)

        if self.namespace:
            for name, module in self.modules[self.namespace].items():
                wrapped = '\n'.join(textwrap.wrap(module.desc, max_width - 40))
                table.table_data.append(
                    [self.namespace + "/" + name, wrapped])
        else:
            for k, v in self.modules.items():
                for name, module in v.items():
                    wrapped = '\n'.join(textwrap.wrap(module.desc, max_width - 40))
                    table.table_data.append(
                        [k + "/" + name, wrapped])

        print(table.table)

    def do_back(self, arg):
        """Go back from namespace or module"""

        if self.module:
            self.module = ""
        else:
            self.namespace = ""

    def do_manage(self, arg):
        """Add, edit or delete module"""

        def save_mod_json(data):
            with open(mod_json_file, 'w') as f:
                json.dump(data, f, indent=4)

        # Read modules.json
        data = get_mod_json()
        try:
            edit_prompt = [
                {
                    'type': 'list',
                    'name': 'option',
                    'message': 'What do you want to do?',
                    'choices': ["Add module", "Edit module", "Remove module"],
                }
            ]
            choice = prompt(edit_prompt)

            # ADD
            if choice['option'] == "Add module":
                # Namespace selection
                ns_prompt = [
                    {
                        'type': 'list',
                        'name': 'namespace',
                        'message': 'Select a namespace for your module',
                        'choices': list(self.modules.keys()) + ["new"],
                    }
                ]
                # Module details
                mod_ns = prompt(ns_prompt)['namespace']
                mod_questions = [
                    {
                        'type': 'input',
                        'name': 'mod_name',
                        'message': 'Name of your module',
                        'validate': ModuleNameValidation,
                    },
                    {
                        'type': 'input',
                        'name': 'mod_desc',
                        'message': 'Module description',
                    },
                    {
                        'type': 'input',
                        'name': 'mod_cmd',
                        'message': 'Command',
                    },
                    {
                        'type': 'confirm',
                        'name': 'mod_su',
                        'message': 'Does this command require superuser?',
                    },
                    {
                        'type': 'confirm',
                        'name': 'mod_rr',
                        'message': 'Does this command require OS / Service restart?'
                    }
                ]
                if mod_ns == "new":
                    mod_question = [{
                        'type': 'input',
                        'name': 'mod_ns',
                        'message': 'Name of your namespace',
                        'validate': NamespaceValidation
                    }]
                    mod_namespace = prompt(mod_question)
                try:
                    mod_ns = mod_namespace['mod_ns']
                except Exception:
                    pass
                # Assigning property to the ModuleNameValidation class to
                # access modules within the selected namespace.
                ModuleNameValidation.modules = self.modules[mod_ns].keys() \
                    if mod_ns in self.modules.keys() else []
                mod_details = prompt(mod_questions)

                # Convert boolean values to strings
                require_superuser_str = "True" if mod_details['mod_su'] else "False"
                require_restart_str = "True" if mod_details['mod_rr'] else "False"

                mod_dict = {
                    "name": mod_details['mod_name'].title(),
                    "desc": mod_details['mod_desc'],
                    "command": mod_details['mod_cmd'],
                    "require_superuser": require_superuser_str,
                    "require_restart": require_restart_str,
                    "target_os": "win" if check_os() else "linux"
                }
                try:
                    data[mod_ns.lower()].append(mod_dict)
                except KeyError:
                    data.update({mod_ns.lower(): [mod_dict]})
                # Write the updated modules.json
                save_mod_json(data)
                logger.info("Module added successfully. Use 'list' " +
                            "command to see available modules.")

            # EDIT & REMOVE
            elif choice['option'] == "Edit module" or choice['option'] == "Remove module":
                mod_option = choice['option'].split(" ")[0].lower()
                # Namespace selection
                ns_prompt = [
                    {
                        'type': 'list',
                        'name': 'namespace',
                        'message': "Select the namespace of module to " + mod_option,
                        'choices': list(self.modules.keys())
                    }
                ]
                selected_ns = prompt(ns_prompt)['namespace']
                # Module selection
                mod_prompt = [
                    {
                        'type': 'list',
                        'name': 'module',
                        'message': "Select a module to " + mod_option,
                        'choices': self.modules[selected_ns]
                    }
                ]
                selected_mod = prompt(mod_prompt)['module']
                mod_list = [mod['name'] for mod in list(data[selected_ns])]
                mod_index = mod_list.index(selected_mod)

                # EDIT
                if mod_option == "edit":
                    # Create a list for properties
                    prop_list = list(self.modules[selected_ns][selected_mod].kwargs.keys())
                    prop_list.remove('namespace')
                    prop_list.remove('target_os')
                    # Module property selection
                    prop_prompt = [
                        {
                            'type': 'list',
                            'name': 'property',
                            'message': "Select a property for editing " + selected_mod,
                            'choices': prop_list
                        }
                    ]
                    selected_prop = prompt(prop_prompt)['property']
                    # New value for property
                    new_val = prompt([{
                        'type': 'input',
                        'name': 'val',
                        'message': "New value for " + selected_prop}])['val']
                    new_val = new_val.title() if selected_prop == "name" else new_val
                    # Update the selected property of module
                    data[selected_ns][mod_index][selected_prop] = new_val
                    # Write the updated modules.json
                    save_mod_json(data)
                    logger.info("Module updated successfully. (" + selected_ns + "/" +
                                selected_mod + ":" + selected_prop + ")")

                # REMOVE
                else:
                    data[selected_ns].pop(mod_index)
                    save_mod_json(data)
                    logger.info("Module removed successfully. (" + selected_mod + ")")
            else:
                pass
        except Exception as e:
            logger.error(str(e))
        self.namespace = ""
        self.module = ""
        self.modules = get_modules()

    def do_preset(self, arg):
        """Show/execute the hardening module presets"""

        presets = get_presets()
        if arg:
            modules = [preset['modules'] for preset in presets
                       if preset['name'] == arg]
            if len(modules) == 0:
                logger.error(f"Preset not found: '{arg}'")
                return
            modules = modules[0]
            confirm_prompt = [
                {
                    'type': 'confirm',
                    'name': 'confirm',
                    'message': 'Run modules without confirmation?',
                }
            ]
            try:
                conf_mod = prompt(confirm_prompt)['confirm']
            except KeyError:
                return
            # Main module loop
            for module in modules:
                # Select the namespace
                self.namespace = module.split("/")[0].lower()
                # Loop through the modules
                for name, mod in self.modules[self.namespace].items():
                    # Select the module if it equals to the module in
                    # the preset or equals to 'all'
                    if module.split("/")[1].lower() == "all" or \
                            module.split("/")[1].lower() == name.lower():
                        # Select the module
                        self.module = str(mod)
                        # Show module information
                        self.do_info(None)
                        # If confirmation not needed
                        if conf_mod:
                            self.do_harden(None)
                        else:
                            # Ask for permission for executing the command
                            exec_conf = prompt([{
                                'type': 'confirm',
                                'name': 'confirm',
                                'message': 'Execute the hardening command?',
                            }])
                            # Execute the command or cancel
                            try:
                                if exec_conf['confirm']:
                                    self.do_harden(None)
                                else:
                                    raise Exception("Cancelled by user.")
                            except Exception:
                                logger.info("Hardening cancelled. " +
                                            f"({self.namespace}/{self.module})")
            # Go back from the selected module and namespace
            self.module = ""
            self.namespace = ""
        else:
            if not presets:
                logger.warn(f"No presets found in {mod_json_file}")
                return
            search_table = [['Preset', 'Modules']]
            table = AsciiTable(search_table)
            max_width = table.column_max_width(1)
            # Create a table of presets
            for preset in presets:
                mods = ""
                for module in preset['modules'][:-1]:
                    mods += '\n'.join(textwrap.wrap(module, max_width - 40)) + '\n'
                mods += '\n'.join(textwrap.wrap(preset['modules'][-1], max_width - 40))
                table.table_data.append([preset['name'], mods])
            # Show the table
            if len(table.table_data) > 1:
                print(table.table)
            else:
                logger.warn(f"No presets found in {mod_json_file}")

    def complete_preset(self, text, line, begidx, endidx):
        """Complete preset command"""

        avb_presets = [i['name'].lower() for i in get_presets()]
        mline = line.lower().partition(' ')[2]
        offs = len(mline) - len(text)
        return [s[offs:].title() for s in avb_presets if s.startswith(mline)]

    def do_web(self, arg):
        """Start the grapheneX web server"""

        from graphenex.core.web import run_server, app
        app.config['ACCESS_TOKEN'] = secrets.token_urlsafe(12)
        run_server({"host_port": arg} if arg else None, False)

    def do_harden(self, arg):
        """Execute the hardening command"""

        namespace_data = get_mod_json()[self.namespace]
        mod_requires_restart = None

        for module in namespace_data:
            if module["name"] == self.module:
                mod_requires_restart = module["require_restart"]
                break

        if mod_requires_restart:
            logger.info(f"{self.module} requires restart of OS or the service itself for the changes to apply.")

        if not (self.module and self.namespace):
            logger.error('Select a module/namespace.')
        else:
            try:
                hrd = self.modules[self.namespace][self.module]
                hrd.execute_command()
                logger.info("Hardening command executed successfully.")
            except PermissionError:
                err_msg = "Insufficient permissions for hardening."
                if check_os():
                    err_msg += " Get admin rights and rerun grapheneX."
                else:
                    err_msg += " Try running grapheneX with sudo."
                logger.error(err_msg)
            except Exception as e:
                logger.error(f"Failed to execute hardening command: {e}")

    def do_EOF(self, arg):
        """EOF exit"""

        print()
        self.do_exit(arg)
        return True

    @staticmethod
    def do_exit(arg):
        """Exit interactive shell"""

        exit_msgs = [
            "Bye!",
            "Hope to see you soon!",
            "Take care!",
            "I am not going to miss you!",
            "Gonna miss you!",
            "Thank God, you're leaving. What a relief!",
            "Fare thee well!",
            "Farewell, boss.",
            "Daha karpuz kesecektik.",
            "Bon voyage!",
            "Regards.",
            "Exiting..."]
        logger.info(random.choice(exit_msgs))
        return True

    @staticmethod
    def do_clear(arg):
        """Clear the terminal"""

        os.system("cls" if check_os() else "clear")

    @staticmethod
    def default(line):
        """Default command"""

        logger.error("Command not found.")


class ModuleNameValidation(Validator):
    def validate(self, document):
        """Validate the module name for the prompt"""

        if not re.match(r'^\w+$', document.text):
            raise ValidationError(
                message='Enter a valid module name',
                cursor_position=len(document.text))
        elif document.text.lower() in [module.lower() for module in self.modules]:
            raise ValidationError(
                message="Try a different name, this module name is not available.",
                cursor_position=len(document.text))


class NamespaceValidation(Validator):

    @staticmethod
    def validate(document):
        """Validate the namespace for the prompt"""

        namespaces = get_forbidden_namespaces()
        if document.text.lower() in [namespace.lower() for namespace in namespaces]:
            raise ValidationError(
                message="You do not have permission to access this namespace.",
                cursor_position=len(document.text))



import re
import json
from functools import wraps

from flask import render_template, session, request, redirect, flash
from flask_socketio import emit

from graphenex.core.web import app, logger, socketio
from graphenex.core.utils.helpers import check_os, get_modules, mod_json_file
from graphenex.core.utils.sysinfo import SysInformation

module_dict = get_modules()
current_namespace = list(module_dict.keys())[0]


def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        token = session.get('token', None)
        if token != app.config['ACCESS_TOKEN']:
            return redirect('/auth')
        else:
            return f(*args, **kwargs)
    return decorated_function


def auth_socketio(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if session.get('token', None) == app.config['ACCESS_TOKEN']:
            return f(*args, **kwargs)
        else:
            emit('auth', {'text': 'Not authenticated'})

    return decorated_function


@app.route('/auth', methods=["GET", "POST"])
def auth():
    if session.get('token', None) == app.config['ACCESS_TOKEN']:
        return redirect('/')

    if request.method == 'POST':
        token = request.form['token']
        if token == app.config['ACCESS_TOKEN']:
            session['token'] = token
            return redirect('/')
        else:
            flash('Token is not valid', 'error')

    return render_template('auth.html')


@app.route('/')
@login_required
def main():
    return render_template(
        'index.html',
        title="grapheneX [Web]",
        mod_count=get_mod_count(module_dict),
        sys_all_info=SysInformation.get_all_info())


@app.errorhandler(404)
def page_not_found(e):
    return render_template('404.html'), 404


def get_mod_count(mod_dict):
    """Return the count of modules in the dict"""
    return sum([len(value) for key, value in mod_dict.items()])


@socketio.on('get_namespaces')
@auth_socketio
def send_namespaces(data):
    """Send namespaces to the web"""
    emit('get_namespaces', {'namespaces': list(module_dict.keys())})


@socketio.on('get_current_namespace')
@auth_socketio
def send_current_namespace(data):
    """Send the current namespace to the web"""
    emit('get_current_namespace', {'current_namespace': current_namespace})


@socketio.on('send_current_namespace')
@auth_socketio
def get_current_namespace(data):
    """Set the current namespace and send modules"""
    modules = list()
    mod_dict = module_dict.get(data)
    if mod_dict is None:
        logger.warn(f"Non-existent namespace: \"{data}\".")
    else:
        global current_namespace
        current_namespace = data
        for name, mod in mod_dict.items():
            modules.append({
                'name': name,
                'desc': mod.desc,
                'source': mod.command
            })
        logger.info(f'Sending modules of \"{current_namespace}\".')
        emit('get_module', modules)


@socketio.on('search_module')
@auth_socketio
def search_module(data):
    """Search module in the module list"""
    result = {name: mod for name, mod in module_dict.get(
        current_namespace).items() if data["query"].lower() in name.lower()}
    payload = list()
    for name, mod in result.items():
        payload.append({
            'name': name,
            'desc': mod.desc,
            'source': mod.command
        })
    emit('search_module', {'result': payload})


@socketio.on('harden')
@auth_socketio
def hardening_exec(data):
    """Execute the hardening module and send its output to web"""
    try:
        logger.info("Executing the hardening command of " +
                    current_namespace + "/" + data)
        hrd = module_dict[current_namespace][data]
        out = hrd.execute_command()
        emit(data + "_log", {"msg": out, "state": "output"})
        success_msg = "Hardening command executed successfully."
        emit('log_message', {
            'tag': 'success',
            'content': success_msg
        })
        emit(data + "_log", {"state": "success"})
        logger.info(success_msg)
    except PermissionError:
        err_msg = "Insufficient permissions for hardening."
        if check_os():
            err_msg += " Get admin rights and rerun grapheneX."
        else:
            err_msg += " Try running grapheneX with sudo."
        emit('log_message', {
            'tag': 'warning',
            'content': err_msg,
            'duration': 2000
        })
        emit(data + "_log", {"state": "error"})
        logger.error(err_msg)
    except Exception as e:
        fail_msg = "Failed to execute hardening command"
        emit('log_message', {
            'tag': 'danger',
            'content': fail_msg,
            'duration': 2000
        })
        emit(data + "_log", {"msg": str(e), "state": "error"})
        logger.error(f"{fail_msg}: {e}")


@socketio.on('add_module')
@auth_socketio
def add_module(mod):
    """Add new module to the framework"""
    try:
        mod_name = mod['name']
        mod_ns = mod['ns']
        logger.info("Adding new module: '" + mod_ns + "/" + mod_name + "'")
        # Check namespace
        if not mod_ns:
            ns_error_msg = "Invalid namespace."
            logger.error(ns_error_msg)
            emit('log_message', {
                'tag': 'danger',
                'content': ns_error_msg
            })
            return
        # Check module name
        if not re.match(r'^\w+$', mod_name):
            mod_error_msg = "Invalid module name."
            logger.error(mod_error_msg)
            emit('log_message', {
                'tag': 'danger',
                'content': mod_error_msg
            })
            return
        # Get modules.json
        with open(mod_json_file, 'r') as f:
            data = json.load(f)
        # Prepare module dict
        mod_dict = {
            "name": mod_name,
            "desc": mod['desc'],
            "command": mod['cmd'],
            "require_superuser": str(mod['su']).capitalize(),
            "require_restart": str(mod['rr']).capitalize(),
            "target_os": "win" if check_os() else "linux"
        }
        # Append to json data
        try:
            data[mod_ns].append(mod_dict)
        except KeyError:
            data.update({mod_ns: [mod_dict]})
        # Update the modules.json
        with open(mod_json_file, 'w') as f:
            json.dump(data, f, indent=4)
        success_msg = "Module added successfully."
        emit('log_message', {
            'tag': 'success',
            'content': success_msg
        })
        logger.info(success_msg)
        global module_dict
        module_dict = get_modules()
        # Success event with module count
        emit('new_module_added', get_mod_count(module_dict))
    except Exception as e:
        exception_msg = f"Error occurred while adding new module: {e}"
        emit('log_message', {
            'tag': 'warning',
            'content': exception_msg,
            'duration': 2000
        })
        logger.warn(exception_msg)



from graphenex.core.web import auth_socketio, socketio, emit
from graphenex.core.utils.sysinfo import SysInformation


@socketio.on('get_network_info')
@auth_socketio
def send_network_info():
    masks = SysInformation.get_network_info()
    emit('get_network_info', masks)



from graphenex.core.hrd.exec import WinExec, LinuxExec


class HardenMethod:
    """
    Class for retrieving the details of a
    module and executing the command
    """

    def __init__(self, **kwargs):
        self.linuxExec = LinuxExec()
        self.winExec = WinExec()
        self.kwargs = kwargs
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.__str__()

    def execute_command(self):
        return getattr(self, self.target_os + "Exec").run_cmd(self.command)



import sys

from graphenex.core.utils.helpers import parse_cli_args, print_header, create_env_file
from graphenex.core.utils.logcl import GraphenexLogger
from graphenex.core.cli.shell import start_cli
from graphenex.core.web import run_server
from graphenex import __version__

logger = GraphenexLogger('Graphenex')


def main():
    args = parse_cli_args()
    if args['version']:
        print(f"grapheneX v{__version__}")
        sys.exit(0)

    create_env_file(args["flask_secret_key"])

    print_header()
    if args['web']:
        run_server(args)
    else:
        if args['open']:
            logger.warn("[--open] argument is unnecessary. " +
                        "Use with [-w] or [--web].")
        start_cli()


if __name__ == "__main__":
    main()



import os
import shlex
import subprocess
from abc import ABC, abstractmethod


class OsExec(ABC):
    @abstractmethod
    def run_cmd(self, cmd):
        pass


class LinuxExec(OsExec):
    def run_cmd(self, cmd, shell=True, **kwargs):
        """
        Executes the Linux command and returns its output in UTF-8 format.
        Supports passing `kwargs`.
        """

        cmd = cmd.replace("$USER", os.environ["USER"])
        try:
            result = subprocess.run(cmd, stdout=subprocess.PIPE, shell=shell, **kwargs)
        except subprocess.CalledProcessError as e:
            raise PermissionError(f"Subprocess err {e} caught while running '{cmd}'")
        else:
            if result.returncode != 0:
                raise PermissionError(f"Command '{cmd}' returned non-zero exit status!")

        try:
            return result.stdout.decode('utf-8')
        except AttributeError:
            return 'stdout was redirected to a file'


class WinExec(OsExec):
    def run_cmd(self, cmd, shell=True):
        """
        Executes the Windows command and returns its output in UTF-8 format.
        """

        result = subprocess.run(shlex.split(
            cmd), stdout=subprocess.PIPE, shell=shell)
        return result.stdout.decode('utf-8', 'replace')
